/*******************************************************************************
*
*  FREESCALE SEMICONDUCTOR INC. 2004
*  ALL RIGHTS RESERVED
*  COPYRIGHT (c)
*  
********************************************************************************
*
*  DESCRIPTION:
*   Hall Decoder eTPU function
*
********************************************************************************
*
*  REVISION HISTORY:
*                                                         
*  REVISION    AUTHOR       DATE        DESCRIPTION OF CHANGE                  
*  --------    -----------  ---------   ------------------------------------                  
*  1.0         V.Phillipov  05/Oct/04   Generation of interrupts added.
*              Milan Brejl              Ready for release.
*              Michal Princ
*******************************************************************************/


#ifndef SET_USED
  #pragma entryaddr 0x0000;		          // Base entry table address
  #define HD_FUNCTION_NUMBER      1     // Function number.
  #include "../set3/etpuc_pwmc.h"       // Commutations
#endif

#include <eTPUc.h>                    //Defines eTPU hardware
#include <eTPUc_common.h>             //Standard way to use eTPU

#pragma ETPU_function HD, alternate @ HD_FUNCTION_NUMBER;

//************************************************************************
//*  Definitions.
//*************************************************************************/

// Host Service Requests
#define HD_HSR_INIT             7
#define HD_HSR_COMMUT_ON        5
#define HD_HSR_COMMUT_OFF       3

// Function Modes
#define HD_DIRECTION_ON   (fm0==1)
#define HD_REV_COUNT_ON   (fm1==1)

// Pin states
#define HD_PIN_HIGH             1
#define HD_PIN_LOW              0

// timer values.
#define HD_TCR1                 0
#define HD_TCR2                 1

/************************************************************************
* Common HD paramers structure type definition.   
************************************************************************/
typedef struct {              
  unsigned int24 sector;         //Number of current sector 
  unsigned int24 direction;      //Direction number    
           int24 rev_counter;    //Revolution counter
  unsigned int24 rev_period;     //Revolution period   
  unsigned int24 sector_period;  //Commutation period  
           int24 last_edge;      //Last edge time      
  unsigned int24 timer;          //ETPU_HD_TCR1 or ETPU_HD_TCR2
} hd_common_param;

/************************************************************************
* NAME: HD
*
* DESCRIPTION: Hall Decoder   
* 
* FUNCTION PARAMETERS:
*
*  sector_mask_LH    -  Sector LH_mask, depends on configuration
*  sector_mask_HL    -  Sector HL_mask, depends on configuration
*  direction_mask_LH -  Direction LH_mask, depends on configuration
*  direction_mask_HL -  Direction HL_masl, depends on configuration
*  comm_cmd_LH_I_0   -  Commutation command to execute on LH transition 
*                       at incremental direction, as first
*  comm_cmd_LH_I_1   -  Commutation command to execute on LH transition
*                       at incremental direction, as second
*  comm_cmd_LH_D_0   -  Commutation command to execute on LH transition
*                       at decremental direction, as first 
*  comm_cmd_LH_D_1   -  Commutation command to execute on LH transition
*                       at decremental direction, as second
*  comm_cmd_HL_I_0   -  Commutation command to execute on HL transition
*                       at incremental direction, as first 
*  comm_cmd_HL_I_1   -  Commutation command to execute on HL transition
*                       at incremental direction, as second
*  comm_cmd_HL_D_0   -  Commutation command to execute on HL transition
*                       at decremental direction, as first 
*  comm_cmd_HL_D_1   -  Commutation command to execute on HL transition
*                       at decremental direction, as second
*  hd_common         -  Pointer to structure of common parameters
*  PWMM_chan         -  PWMM channel number
*  
*************************************************************************/
void HD(  unsigned int24 sector_mask_LH,     //Sector LH_mask, depends on configuration
          unsigned int24 sector_mask_HL,     //Sector HL_mask, depends on configuration
          unsigned int24 direction_mask_LH,  //Direction LH_mask, depends on configuration
          unsigned int24 direction_mask_HL,  //Direction HL_masl, depends on configuration
          unsigned int32 comm_cmd_LH_I_0,    //Commutation command to execute on LH transition at incremental direction, as first
          unsigned int32 comm_cmd_LH_I_1,    //Commutation command to execute on LH transition at incremental direction, as second
          unsigned int32 comm_cmd_LH_D_0,    //Commutation command to execute on LH transition at decremental direction, as first
          unsigned int32 comm_cmd_LH_D_1,    //Commutation command to execute on LH transition at decremental direction, as second
          unsigned int32 comm_cmd_HL_I_0,    //Commutation command to execute on HL transition at incremental direction, as first
          unsigned int32 comm_cmd_HL_I_1,    //Commutation command to execute on HL transition at incremental direction, as second
          unsigned int32 comm_cmd_HL_D_0,    //Commutation command to execute on HL transition at decremental direction, as first
          unsigned int32 comm_cmd_HL_D_1,    //Commutation command to execute on HL transition at decremental direction, as second
          hd_common_param* hd_common,        //Pointer to structure of common parameters
          int8 PWMM_chan                     //PWMM channel number
)
{
  static int24 last_edge_LH;                 //Last LH edge time for current channel
  static int24 last_edge_HL;                 //Last HL edge time for current channel
         int24 tmp;

/*--------------------------------------------------------------------------+
| THREAD NAME: INIT_COMMUT                                                  |
| DESCRIPTION: Initialize a channel to run without commutation.             |
+--------------------------------------------------------------------------*/
  if ( hsr==HD_HSR_INIT )
  {
    //Configure the channel
    if (hd_common->timer==HD_TCR1) 
    {
      ActionUnitA( MatchTCR1, CaptureTCR1, GreaterEqual);      
      OnTransA(AnyTrans);
      SingleMatchSingleTransition();
      tmp = tcr1;
    }
    else 
    {
      ActionUnitA( MatchTCR2, CaptureTCR2, GreaterEqual);      
      OnTransA(AnyTrans);
      SingleMatchSingleTransition();
      tmp = tcr2;
    }
    Clear(flag1);
    //Initialize time values
    last_edge_HL = tmp;
    last_edge_LH = tmp;
    hd_common->last_edge = tmp;
    //Initialize other values 
    if (CurrentInputPin == 0) 
    {
      Clear(flag0);
      EnableEventHandling();
	    hd_common->sector = hd_common->sector & sector_mask_HL;
    }
    else
    {
      Set(flag0);
      EnableEventHandling();
      hd_common->sector = hd_common->sector | sector_mask_LH;
    }
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: COMMUT_ON                                                    |
| DESCRIPTION: Enable commutations.                                         |
+--------------------------------------------------------------------------*/
  else if ( hsr==HD_HSR_COMMUT_ON )
  {
    Set(flag1);
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: COMMUT_OFF                                                   |
| DESCRIPTION: Disable commutations.                                        |
+--------------------------------------------------------------------------*/
  else if ( hsr==HD_HSR_COMMUT_OFF )
  {
    Clear(flag1);
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: RISING TRANSITION (LH) with commutation                      |
| DESCRIPTION: 1. Calculate direction (if activated)                        |
|              2. Apply commutation commands                                |
|              3. Calculate HD parameters                                   |
|              4. Increment Revolution Counter (if activated in this phase) |
+--------------------------------------------------------------------------*/
  else if( (m2==1) && (pin==1) && (flag0==0) && (flag1==1))
  {
    //Calculculate Direction if activated
    if (HD_DIRECTION_ON)
    {
      hd_common->direction = hd_common->sector & direction_mask_LH;
    }
    //Perform Commutations if activated
    if ( hd_common->direction == 0 )
    {
      //Commutate(comm_cmd_LH_I_0);
      //Commutate(comm_cmd_LH_I_1);
      comm_cmd = comm_cmd_LH_I_0;
      Commutate();
      comm_cmd = comm_cmd_LH_I_1;
      Commutate();
    } 
    else
    {
      //Commutate(comm_cmd_LH_D_0);
      //Commutate(comm_cmd_LH_D_1);
      comm_cmd = comm_cmd_LH_D_0;
      Commutate();
      comm_cmd = comm_cmd_LH_D_1;
      Commutate();
    } 
    // force PWMM channel to reload (apply new phase options)
    tmp = GetCurrentChanNum();
    SwitchToChannel(PWMM_chan);
    Set(flag0);
    SwitchToChannel(tmp);
    
    goto HD_LH;
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: RISING TRANSITION (LH) without commutation                   |
| DESCRIPTION: 1. Calculate direction (if activated)                        |
|              2. Calculate HD parameters                                   |
|              3. Increment Revolution Counter (if activated in this phase) |
+--------------------------------------------------------------------------*/
  else if( (m2==1) && (pin==1) && (flag0==0) && (flag1==0))
  {
    //Calculculate Direction if activated
    if (HD_DIRECTION_ON)
    {
      hd_common->direction = hd_common->sector & direction_mask_LH;
    }
HD_LH:
    //Calculation of HD parametres 
    hd_common->sector_period = erta - hd_common->last_edge;
    hd_common->last_edge = erta;
    hd_common->rev_period = erta - last_edge_LH;
    last_edge_LH = erta;
    hd_common->sector = hd_common->sector | sector_mask_LH;
    //Calculate Revolution counter if activated
    if (HD_REV_COUNT_ON)
    {
      if (hd_common->direction == 0)
      {
        hd_common->rev_counter++;
      }
    }
    Clear(TransLatch);
    Set(flag0);
    SetChannelInterrupt();
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: FALLING TRANSITION (HL) with commutation                     |
| DESCRIPTION: 1. Calculate direction (if activated)                        |
|              2. Apply commutation commands                                |
|              3. Calculate HD parameters                                   |
|              4. Decrement Revolution Counter (if activated in this phase) |
+--------------------------------------------------------------------------*/
  else if( (m2==1) && (pin==0) && (flag0==1) && (flag1==1))
  {
    //Calculculate direction if activated
    if (HD_DIRECTION_ON)
    {
      hd_common->direction = hd_common->sector & direction_mask_HL;
    }
    // Commutations
    if ( hd_common->direction == 0 )
    {
      //Commutate(comm_cmd_HL_I_0);
      //Commutate(comm_cmd_HL_I_1);
      comm_cmd = comm_cmd_HL_I_0;
      Commutate();
      comm_cmd = comm_cmd_HL_I_1;
      Commutate();
    } 
    else
    {
      //Commutate(comm_cmd_HL_D_0);
      //Commutate(comm_cmd_HL_D_1);
      comm_cmd = comm_cmd_HL_D_0;
      Commutate();
      comm_cmd = comm_cmd_HL_D_1;
      Commutate();
    } 
    // force PWMM channel to reload (apply new phase options)
    tmp = GetCurrentChanNum();
    SwitchToChannel(PWMM_chan);
    Set(flag0);
    SwitchToChannel(tmp);
    
    goto HD_HL;
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: FALLING TRANSITION (HL) without commutation                  |
| DESCRIPTION: 1. Calculate direction (if activated)                        |
|              2. Calculate HD parameters                                   |
|              3. Decrement Revolution Counter (if activated in this phase) |
+--------------------------------------------------------------------------*/
  else if( (m2==1) && (pin==0) && (flag0==1) && (flag1==0))
  {
    //Calculculate direction if activated
    if (HD_DIRECTION_ON)
    {
      hd_common->direction = hd_common->sector & direction_mask_HL;
    }
HD_HL:
    //Calculate HD parametres 
    hd_common->sector_period = erta - hd_common->last_edge;
    hd_common->last_edge = erta;
    hd_common->rev_period = erta - last_edge_HL;
    last_edge_HL = erta;
    hd_common->sector = hd_common->sector & sector_mask_HL;
    //Calculate Revolution counter if activated
    if (HD_REV_COUNT_ON)
    {
      if (hd_common->direction > 0)
      {
        hd_common->rev_counter--;
      }
    }
    Clear(TransLatch);
    Clear(flag0);
    SetChannelInterrupt();
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: NOISE TRANSITION                                             |
| DESCRIPTION: 1. Enable next transition detections                         |
+--------------------------------------------------------------------------*/
  else if( ((m2==1) && (pin==1) && (flag0==1)) || ((m2==1) && (pin==0) && (flag0==0)) )
  {
    Clear(TransLatch);
  }
  else
  {
#ifdef SET_USED
    Global_Error_Func();           // Call the Global error routine and end.
#endif
  }
}

/* Information exported to Host CPU program */
#pragma write h, (::ETPUfilename (cpu/etpu_hd_auto.h));
#pragma write h, ( );
#pragma write h, (/****************************************************************/);
#pragma write h, (/* WARNING this file is automatically generated DO NOT EDIT IT! */);
#pragma write h, (/*                                                              */);
#pragma write h, (/* This file provides an interface between eTPU code and CPU    */);
#pragma write h, (/* code. All references to the PWM function should be made with */);
#pragma write h, (/* information in this file. This allows only symbolic          */);
#pragma write h, (/* information to be referenced which allows the eTPU code to be*/);
#pragma write h, (/* optimized without effecting the CPU code.                    */);
#pragma write h, (/****************************************************************/);
#pragma write h, (#ifndef _ETPU_HD_AUTO_H_ );
#pragma write h, (#define _ETPU_HD_AUTO_H_ );
#pragma write h, ( );
#pragma write h, (/* Function Configuration Information */);
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_FUNCTION_NUMBER)        HD_FUNCTION_NUMBER );
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_TABLE_SELECT)           ::ETPUentrytype(HD) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_NUM_PARMS)              ::ETPUram(HD) );
#pragma write h, ( );
#pragma write h, (/* Host Service Request Definitions */);
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_HSR_INIT)               HD_HSR_INIT );
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_HSR_COMMUT_ON)          HD_HSR_COMMUT_ON );
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_HSR_COMMUT_OFF)         HD_HSR_COMMUT_OFF );
#pragma write h, ( );
#pragma write h, (/* Parameter Definitions */);
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_SECTORMASKLH_OFFSET)    ::ETPUlocation(HD, sector_mask_LH   ));  
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_SECTORMASKHL_OFFSET)    ::ETPUlocation(HD, sector_mask_HL   )); 
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_DIRECTIONMASKLH_OFFSET) ::ETPUlocation(HD, direction_mask_LH));
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_DIRECTIONMASKHL_OFFSET) ::ETPUlocation(HD, direction_mask_HL));
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_COMMCMDLHI0_OFFSET)     ::ETPUlocation(HD, comm_cmd_LH_I_0  ));
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_COMMCMDLHI1_OFFSET)     ::ETPUlocation(HD, comm_cmd_LH_I_1  ));
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_COMMCMDLHD0_OFFSET)     ::ETPUlocation(HD, comm_cmd_LH_D_0  ));
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_COMMCMDLHD1_OFFSET)     ::ETPUlocation(HD, comm_cmd_LH_D_1  ));
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_COMMCMDHLI0_OFFSET)     ::ETPUlocation(HD, comm_cmd_HL_I_0  ));
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_COMMCMDHLI1_OFFSET)     ::ETPUlocation(HD, comm_cmd_HL_I_1  ));
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_COMMCMDHLD0_OFFSET)     ::ETPUlocation(HD, comm_cmd_HL_D_0  ));
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_COMMCMDHLD1_OFFSET)     ::ETPUlocation(HD, comm_cmd_HL_D_1  ));
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_COMMON_STR_OFFSET)      ::ETPUlocation(HD, hd_common        ));
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_PWMMCHAN_OFFSET)        ::ETPUlocation(HD, PWMM_chan        ));
#pragma write h, ( );
#pragma write h, (/* Structure Size Definitions */);
#pragma write h, (::ETPUliteral(#define FS_ETPU_HD_COMMON_STR_SIZE)         ::ETPUsizeof(hd_common));
#pragma write h, ( );
#pragma write h, (#endif );
#pragma write h, ( );

/*********************************************************************
 *
 * Copyright:
 *	Freescale Semiconductor, INC. All Rights Reserved.
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Freescale
 *  Semiconductor, Inc. This software is provided on an "AS IS"
 *  basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, Freescale
 *  Semiconductor DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED,
 *  INCLUDING IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 *  PARTICULAR PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH
 *  REGARD TO THE SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF)
 *  AND ANY ACCOMPANYING WRITTEN MATERIALS.
 *
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL Freescale Semiconductor BE LIABLE FOR ANY DAMAGES WHATSOEVER
 *  (INCLUDING WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
 *  BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER
 *  PECUNIARY LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.
 *
 *  Freescale Semiconductor assumes no responsibility for the
 *  maintenance and support of this software
 ********************************************************************/
