/*******************************************************************************
*
*  FREESCALE SEMICONDUCTOR INC. 2004
*  ALL RIGHTS RESERVED
*  COPYRIGHT (c)
*  
********************************************************************************
*                                                                      
*  DESCRIPTION:
*   The Motor-Control PWM - Commutation Capable (PWMC) function drives
*   a PWM phase with the feature to commutate (switch ON/OFF, change  
*   polarity, swap-channels).                                         
*   This function is not functional standalone. The PWMM (PWM Master) 
*   function must update the PWMC edge times.                         
*
********************************************************************************
*
*  REVISION HISTORY:
*                                                         
*  REVISION    AUTHOR       DATE        DESCRIPTION OF CHANGE                  
*  --------    -----------  ---------   ------------------------------------                  
*   1.0        M. Brejl     20/Sep/04   Ready for set3 release.
*
*******************************************************************************/


#ifndef SET_USED
  #pragma entryaddr 0x0000;		          // Base entry table address
  #define PWMC_FUNCTION_NUMBER    1     // Function Number
  #include "etpuc_pwmm.h"     //shared by PWMM, PWMF and PWMC
  #include "etpuc_pwmc.h"     //commutations
#endif

#include <eTPUc.h>          //Defines eTPU hardware
#include <eTPUc_common.h>   //Standard way to use eTPU

#pragma ETPU_function PWMC, standard @ PWMC_FUNCTION_NUMBER;

//************************************************************************
//*  Definitions.
//*************************************************************************/

// HSR values.
#define PWMC_HSR_INIT              7
#define PWMC_HSR_UPDATE            1
#define PWMC_HSR_STOP              2
#define PWMC_HSR_COMMUTATE         3

// Function Modes
#define PWMC_BASE_CHAN             (fm0==0)
#define PWMC_COMPLEMENTARY_CHAN    (fm0==1)
#define PWMC_ACTIVE_HIGH           (fm1==0)
#define PWMC_ACTIVE_LOW            (fm1==1)

// Channel Flag usage
// Flag0 is used to define which edge should occur next. The problem is that
//    for very small high or low pulses both matches can get set and they
//    need to be serviced in a particular order.
//    0 = MatchA next and 1 = MatchB next.

/*--------------------------------------------------------------------------+
| FUNCTION    : Commutate                                                   |
| PURPOSE     : Executes a commutation command.                             |
| INPUT NOTES : This function has one input parameter:                      |
|               comm_cmd - commutation command. The commutation command     |
|                          consists of 4 parts:                             |
|                          - base channel number (msb byte)                 |
|                          - base channel command (high byte)               |
|                          - complementary channel command (middle byte)    |
|                          - phase options (low byte)                       |
|                          The channel command is one of:                   |
|                            PWMC_COMM_DO_NOTHING                           |
|                            PWMC_COMM_OFF_LOW                              |
|                            PWMC_COMM_OFF_HIGH                             |
|                            PWMC_COMM_ON_ACTIVE_HIGH                       |
|                            PWMC_COMM_ON_ACTIVE_LOW                        |
|                          The phase options rewrites the PWMC options      |
|                          parameter.                                       |
+--------------------------------------------------------------------------*/
//void Commutate( comm_cmd_t comm_cmd )
//void Commutate( registerp comm_cmd )
void Commutate( void)  // 32-bit comm_cmd is passed through register p
{
  register_sr comm_byte;
  register_b compl_ch_comm_byte;
  register_c original_chan;
  register_d compl_done;

  compl_done = 0;
  original_chan = GetCurrentChanNum();
  SwitchToChannel(comm_cmd.b.base_ch_num);
#asm
  // set phase_options to channel parameter 0 msb
  ram p31_0 <- comm_cmd.
  ram p31_24 -> pram0, p_acess_msb.
#endasm
  comm_byte = comm_cmd.b.base_ch_comm_byte;
  compl_ch_comm_byte = comm_cmd.b.compl_ch_comm_byte;
PWMC_COMM:
  switch ( comm_byte )
  {
    case PWMC_COMM_OFF_LOW:
      Pin(SetLow);
      OnMatchA(NoChange);
      OnMatchB(NoChange);
      break;
    case PWMC_COMM_OFF_HIGH:
      Pin(SetHigh);
      OnMatchA(NoChange);
      OnMatchB(NoChange);
      break;
    case PWMC_COMM_ON_ACTIVE_HIGH:
      OnMatchA(PinHigh);
      OnMatchB(PinLow);
      chan = chan;
      if (erta > ertb) // active time
      {
        Pin(SetHigh);
        chan = chan;
        if (erta < ertb)
        {
          Pin(SetLow);
        }
      }
      else             // inactive time
      {
        Pin(SetLow);
        chan = chan;
        if (erta > ertb)
        {
          Pin(SetHigh);
        }
      }
      break;
    case PWMC_COMM_ON_ACTIVE_LOW:
      OnMatchA(PinLow);
      OnMatchB(PinHigh);
      chan = chan;
      if (erta > ertb) // active time
      {
        Pin(SetLow);
        chan = chan;
        if (erta < ertb)
        {
          Pin(SetHigh);
        }
      }
      else             // inactive time
      {
        Pin(SetHigh);
        chan = chan;
        if (erta > ertb)
        {
          Pin(SetLow);
        }
      }
    case PWMC_COMM_DO_NOTHING:
  }
  comm_byte = compl_done;  // to avoid ETPUC b74 bug
  if ( compl_done != 0 ) 
  {
    SwitchToChannel(original_chan);
    return;
  }
  else
  {
    compl_done = 1;
    SwitchToChannel(GetCurrentChanNum()+1);
    comm_byte = compl_ch_comm_byte;
    goto PWMC_COMM;
  }
}



/*--------------------------------------------------------------------------+
| FUNCTION PARAMETERS:                                                      |
| options: Option bits. - bit0=0  -                                        |
| (used by PWMM)          bit0=1  negate duty-cycle                        |
|                       - bit1=0  -                                        |
|                         bit1=1  swap dead-time insertion                 |
| period_copy: Local copy of the PWMM period parameter.                     |
| dtc:     Dead-time correction. - 0 = base channel subtracts deat-times    |
| (used by PWMM)                 - 1 = both channels dead-times/2.          |
|                                - 2 = bottom channel adds dead-times.      |
| A_time:  Calculated base channel match A time.                            |
|          On each match updated for the next period.                       |
| B_time:  Calculated base channel match B time.                            |
|          On each match updated for the next period.                       |
| Ac_time: Calculated complementary channel match A time.                   |
|          On each match updated for the next period.                       |
| Bc_time: Calculated complementary channel match B time.                   |
|          On each match updated for the next period.                       |
| flags: Inner flags written by PWMM. - bit0=0  -                          |
|                                       bit0=1  half_cycle reload          |
|                                     - bit1=0  -                          |
|                                       bit1=1  0% duty-cycle              |
|                                     - bit2=0  -                          |
|                                       bit2=1  100% duty-cycle            |
| comm_cmd: Commutation command to execute by PWMC_HSR_COMMUTATE.           |
|                                                                           |
| STATIC LOCAL VARIABLES:                                                   |
| -                                                                         |
+--------------------------------------------------------------------------*/
void PWMC ( int8  options,
            int24 period_copy,
            int8  dtc,
            int24 A_time, int24 B_time, int24 Ac_time, int24 Bc_time,
            int8  flags)
{
/*--------------------------------------------------------------------------+
| THREAD NAME: INIT                                                         |
| DESCRIPTION: Initialize a channel to run the PWMC function.               |
|              1. Clear latches.                                            |
|              2. Select EitherMatch-NonBlocking-SingleTransition chan.mode.|
|              3. Configure channel to use TCR1 clock.                      |
|              4. Configure input pin actions to no_detect.                 |
|              5. Enable match and transition event handling.               |
|              6. Set pin.                                                  |
|              7. Put equal values into capture registers.                  |
|              8. Clear flag0.                                              |
|              9. Clear latches.                                            |
+--------------------------------------------------------------------------*/
  if (hsr == PWMC_HSR_INIT)
  {
    ClearAllLatches();
    EitherMatchNonBlockingSingleTransition();
    ActionUnitA( MatchTCR1, CaptureTCR1, GreaterEqual);
    ActionUnitB( MatchTCR1, CaptureTCR1, GreaterEqual);
    EnableOutputBuffer();
    OnTransA( NoDetect );
    OnTransB( NoDetect );
    OnMatchA( NoChange);
    OnMatchB( NoChange);
    EnableEventHandling();
    if ( PWMC_ACTIVE_LOW )
    {
      Pin(SetHigh)  ;
    }
    else
    {
      Pin(SetLow);
    }
    erta = tcr1;
    ertb = tcr1;
    WriteErtAToMatchAAndEnable();
    WriteErtBToMatchBAndEnable();
    Clear(flag0);
    ClearAllLatches();
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: STOP                                                         |
| DESCRIPTION: Stop the PWM generation, set pin to inactive state.          |
|              1. Disable matches in thread in case there are any previous  |
|                  pending matches. This could happen if the channel was    |
|                  running another function before this one.                |
|              2. Disable match and transition event handling.              |
|              3. Disable match detection.                                  |
|              4. Clear latches.                                            |
|              5. Set pin to inactive state.                                |
+--------------------------------------------------------------------------*/
  else if (hsr == PWMC_HSR_STOP )
  {
    DisableMatchesInThread();
    DisableEventHandling();
    DisableMatchDetection();
    ClearAllLatches();
    Pin(PerPacB);
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: COMMUTATE                                                    |
| DESCRIPTION: Execute the commutation command.                             |
|              1. Executes the commutation command.                         |
+--------------------------------------------------------------------------*/
  else if (hsr == PWMC_HSR_COMMUTATE )
  {
    Commutate();
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: UPDATE_0                                                     |
| DESCRIPTION: Reschedule :                                                 |
|              - in normal upd: Match A and                                 |
|                               Match B.                                    |
|              - in half-cycle: Match B.                                    |
|              1. If not half-cycle realod                                  |
|                   On base channel:                                        |
|                   - schedule match A at A_time.                           |
|                   - update A_time = A_time + period_copy.                 |
|                   On complementary channel:                               |
|                   - schedule match A at Ac_time.                          |
|                   - update Ac_time = Ac_time + period_copy.               |
|                   If (flag 0% set), set OPAC A = no_change,               |
|                   else set OPAC A according to channel polarity.          |
|              2. On base channel:                                          |
|                 - schedule match B at B_time.                             |
|                 - update B_time = B_time + period_copy.                   |
|                 On complementary channel:                                 |
|                 - schedule match B at Bc_time.                            |
|                 - update Bc_time = Bc_time + period_copy.                 |
|                 If (flag 100% set), set OPAC B = no_change,               |
|                 else set OPAC B according to channel polarity.            |
|              3. Clear link service latch.                                 |
+--------------------------------------------------------------------------*/
  else if ( ((lsr == 1)               && (flag0 == 0)) ||
            ((hsr == PWMC_HSR_UPDATE) && (flag0 == 0)) )
  {
    Clear(lsr);
    if ( !(flags & PWMM_HALF_CYCLE) )
    {
PWMC_AB:
      if ( PWMC_BASE_CHAN )
      {
        erta = A_time;              // Setup Match A, but do not clear mrlA
        EnableMatchA();
        A_time = erta + period_copy;// predict for next period
      }
      else
      {
        erta = Ac_time;             // Setup Match A, but do not clear mrlA
        EnableMatchA();
        Ac_time = erta + period_copy;// predict for next period
      }
    }
    goto PWMC_B;
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: UPDATE_1                                                     |
| DESCRIPTION: Reschedule :                                                 |
|              - in normal upd: Match A.                                    |
|              - in half-cycle: Match A and                                 |
|                               Match B.                                    |
|              1. On base channel:                                          |
|                 - schedule match A at A_time.                             |
|                 - update A_time = A_time + period_copy.                   |
|                 On complementary channel:                                 |
|                 - schedule match A at Ac_time.                            |
|                 - update Ac_time = Ac_time + period_copy.                 |
|                 If (flag 0% set), set OPAC A = no_change,                 |
|                 else set OPAC A according to channel polarity.            |
|              2. If half-cycle reload                                      |
|                   On base channel:                                        |
|                   - schedule match B at B_time.                           |
|                   - update B_time = B_time + period_copy.                 |
|                   On complementary channel:                               |
|                   - schedule match B at Bc_time.                          |
|                   - update Bc_time = Bc_time + period_copy.               |
|                   If (flag 100% set), set OPAC B = no_change,             |
|                   else set OPAC B according to channel polarity.          |
|              3. Clear link service latch.                                 |
+--------------------------------------------------------------------------*/
  else if ( ((lsr == 1)               && (flag0 == 1)) ||
            ((hsr == PWMC_HSR_UPDATE) && (flag0 == 1)) )
  {
    Clear(lsr);
    if ( flags & PWMM_HALF_CYCLE )
      goto PWMC_AB;
    else
      goto PWMC_A;
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: MATCH_A                                                      |
| DESCRIPTION: Schedule match A at on A_time (resp. Ac_time) and            |
|              update A_time (resp. Ac_time) for next period.               |
|              1. On base channel:                                          |
|                 - schedule match A at A_time.                             |
|                 - update A_time = A_time + period_copy.                   |
|                 On complementary channel:                                 |
|                 - schedule match A at Ac_time.                            |
|                 - update Ac_time = Ac_time + period_copy.                 |
|              2. If (flag 0% set), set OPAC A = no_change,                 |
|                 else set OPAC A according to channel polarity.            |
|              3. Set flag0.                                                |
+--------------------------------------------------------------------------*/
  else if (m1 == 1)
  {
    ClearMatchALatch();
    Set(flag0);
PWMC_A:
    if ( PWMC_BASE_CHAN )
    {
      erta = A_time;
      EnableMatchA();
      A_time = erta + period_copy;     // predict for next period
    }
    else
    {
      erta = Ac_time;
      EnableMatchA();
      Ac_time = erta + period_copy;    // predict for next period
    }
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: MATCH_B                                                      |
| DESCRIPTION: Schedule match B at on B_time (resp. Bc_time) and            |
|              update B_time (resp. Bc_time) for next period.               |
|              1. On base channel:                                          |
|                 - schedule match B at B_time.                             |
|                 - update B_time = B_time + period_copy.                   |
|                 On complementary channel:                                 |
|                 - schedule match B at Bc_time.                            |
|                 - update Bc_time = Bc_time + period_copy.                 |
|              2. If (flag 100% set), set OPAC B = no_change,               |
|                 else set OPAC B according to channel polarity.            |
|              3. Clear flag0.                                              |
+--------------------------------------------------------------------------*/
  else if (m2 == 1)
  {
    ClearMatchBLatch();
    Clear(flag0);
PWMC_B:
    if ( PWMC_BASE_CHAN )
    {
      ertb = B_time;
      EnableMatchB();
      B_time = ertb + period_copy;     // predict for next period
    }
    else
    {
      ertb = Bc_time;
      EnableMatchB();
      Bc_time = ertb + period_copy;    // predict for next period
    }
  }
/*--------------------------------------------------------------------------+
| Catch all unspecified entry table conditions                              |
+--------------------------------------------------------------------------*/
  else
  {
      #ifdef SET_USED
      Global_Error_Func();           // Call the Global error routine and end.
      #endif
  }
}

/* Information exported to Host CPU program */
#pragma write h, (::ETPUfilename (cpu/etpu_pwmc_auto.h));
#pragma write h, ( );
#pragma write h, (/****************************************************************/);
#pragma write h, (/* WARNING this file is automatically generated DO NOT EDIT IT! */);
#pragma write h, (/*                                                              */);
#pragma write h, (/* This file provides an interface between eTPU code and CPU    */);
#pragma write h, (/* code. All references to the PWMC function should be made with*/);
#pragma write h, (/* information in this file. This allows only symbolic          */);
#pragma write h, (/* information to be referenced which allows the eTPU code to be*/);
#pragma write h, (/* optimized without effecting the CPU code.                    */);
#pragma write h, (/****************************************************************/);
#pragma write h, (#ifndef _ETPU_PWMC_AUTO_H_ );
#pragma write h, (#define _ETPU_PWMC_AUTO_H_ );
#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Function Configuration Information. );
#pragma write h, (****************************************************************/);
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_FUNCTION_NUMBER) PWMC_FUNCTION_NUMBER );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_TABLE_SELECT) ::ETPUentrytype(PWMC) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_NUM_PARMS) ::ETPUram(PWMC) );
#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Host Service Request Definitions. );
#pragma write h, (****************************************************************/);
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_INIT       ) PWMC_HSR_INIT );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_UPDATE     ) PWMC_HSR_UPDATE );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_STOP       ) PWMC_HSR_STOP );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_COMMUTATE  ) PWMC_HSR_COMMUTATE );
#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Parameter Definitions. );
#pragma write h, (****************************************************************/);
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_OPTIONS_OFFSET    ) ::ETPUlocation (PWMC, options) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_PERIODCOPY_OFFSET ) ::ETPUlocation (PWMC, period_copy) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_DTC_OFFSET        ) ::ETPUlocation (PWMC, dtc) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_ATIME_OFFSET      ) ::ETPUlocation (PWMC, A_time) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_BTIME_OFFSET      ) ::ETPUlocation (PWMC, Ac_time) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_ACTIME_OFFSET     ) ::ETPUlocation (PWMC, B_time) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_BCTIME_OFFSET     ) ::ETPUlocation (PWMC, Bc_time) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_FLAGS_OFFSET      ) ::ETPUlocation (PWMC, flags) );
#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Globals Definitions. );
#pragma write h, (****************************************************************/);
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMC_COMMCMD_OFFSET    ) ::ETPUlocation (comm_cmd) );
#pragma write h, ( );
#pragma write h, (#endif );
#pragma write h, ( );


/*********************************************************************
 *
 * Copyright:
 *	Freescale Semiconductor, INC. All Rights Reserved.
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Freescale
 *  Semiconductor, Inc. This software is provided on an "AS IS"
 *  basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, Freescale
 *  Semiconductor DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED,
 *  INCLUDING IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 *  PARTICULAR PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH
 *  REGARD TO THE SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF)
 *  AND ANY ACCOMPANYING WRITTEN MATERIALS.
 *
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL Freescale Semiconductor BE LIABLE FOR ANY DAMAGES WHATSOEVER
 *  (INCLUDING WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
 *  BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER
 *  PECUNIARY LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.
 *
 *  Freescale Semiconductor assumes no responsibility for the
 *  maintenance and support of this software
 ********************************************************************/
