/*******************************************************************************
*
*  FREESCALE SEMICONDUCTOR INC. 2004
*  ALL RIGHTS RESERVED
*  COPYRIGHT (c)
*  
********************************************************************************
*
*  DESCRIPTION:
*   The Motor-Control PWM - Full Range (PWMF) function drives a PWM phase
*   with the feature of full 0%-100% duty-cycle range.                   
*   This function is not functional standalone. The PWMM (PWM Master)    
*   function must update the PWMF edge times.                            
*
*
********************************************************************************
*
*  REVISION HISTORY:
*                                                         
*  REVISION    AUTHOR       DATE        DESCRIPTION OF CHANGE                  
*  --------    -----------  ---------   ------------------------------------                  
*   1.0        M. Brejl     12/Oct/04   Ready for set3 release.
*
*******************************************************************************/

#ifndef SET_USED
  #pragma entryaddr 0x0000;		          // Base entry table address
  #define PWMF_FUNCTION_NUMBER    1     // Function Number
  #include "etpuc_pwmm.h"     //shared by PWMM, PWMF and PWMC
#endif

#include <eTPUc.h>          //Defines eTPU hardware
#include <eTPUc_common.h>   //Standard way to use eTPU

#pragma ETPU_function PWMF, standard @ PWMF_FUNCTION_NUMBER;

//************************************************************************
//*  Definitions.
//*************************************************************************/

// Host service requests
#define PWMF_HSR_INIT              7
#define PWMF_HSR_UPDATE            1
#define PWMF_HSR_STOP              2

// Function Modes
#define PWMF_BASE_CHAN             (fm0==0)
#define PWMF_COMPLEMENTARY_CHAN    (fm0==1)
#define PWMF_ACTIVE_HIGH           (fm1==0)
#define PWMF_ACTIVE_LOW            (fm1==1)

// Channel Flag usage
// Flag0 is used to define which edge should occur next. The problem is that
//    for very small high or low pulses both matches can get set and they
//    need to be serviced in a particular order.
//    0 = MatchA next and 1 = MatchB next.

/*--------------------------------------------------------------------------+
| FUNCTION PARAMETERS:                                                      |
| options: Option bits. - bit0=0 … -                                        |
| (used by PWMM)          bit0=1 … negate duty-cycle                        |
|                       - bit1=0 … -                                        |
|                         bit1=1 … swap dead-time insertion                 |
| period_copy: Local copy of the PWMM period parameter.                     |
| dtc:     Dead-time correction. - 0 = base channel subtracts deat-times    |
| (used by PWMM)                 - 1 = both channels dead-times/2.          |
|                                - 2 = bottom channel adds dead-times.      |
| A_time:  Calculated base channel match A time.                            |
|          On each match updated for the next period.                       |
| B_time:  Calculated base channel match B time.                            |
|          On each match updated for the next period.                       |
| Ac_time: Calculated complementary channel match A time.                   |
|          On each match updated for the next period.                       |
| Bc_time: Calculated complementary channel match B time.                   |
|          On each match updated for the next period.                       |
| flags: Inner flags written by PWMM. - bit0=0 … -                          |
|                                       bit0=1 … half_cycle reload          |
|                                     - bit1=0 … -                          |
|                                       bit1=1 … 0% duty-cycle              |
|                                     - bit2=0 … -                          |
|                                       bit2=1 … 100% duty-cycle            |
|                                                                           |
| STATIC LOCAL VARIABLES:                                                   |
| -                                                                         |
+--------------------------------------------------------------------------*/
void PWMF ( int8  options,
            int24 period_copy,
            int8  dtc,
            int24 A_time, int24 B_time, int24 Ac_time, int24 Bc_time,
            int8  flags )
{
/*--------------------------------------------------------------------------+
| THREAD NAME: INIT                                                         |
| DESCRIPTION: Initialize a channel to run the PWMF function.               |
|              1. Disable matches in thread in case there are any previous  |
|                  pending matches. This could happen if the channel was    |
|                  running another function before this one.                |
|              2. Select EitherMatch-NonBlocking-SingleTransition chan.mode.|
|              3. Configure channel to use TCR1 clock.                      |
|              4. Configure input pin actions to no_detect.                 |
|              5. Configure output pin actions according to polarity.       |
|              6. Enable match and transition event handling.               |
|              7. Clear flag0.                                              |
|              8. Set pin to inactive state.                                |
|              9. Clear latches.                                            |
+--------------------------------------------------------------------------*/
  if (hsr == PWMF_HSR_INIT)
  {
    DisableMatchesInThread();
    EitherMatchNonBlockingSingleTransition();
    ActionUnitA( MatchTCR1, CaptureTCR1, GreaterEqual);
    ActionUnitB( MatchTCR1, CaptureTCR1, GreaterEqual);
    EnableOutputBuffer();
    OnTransA( NoDetect );
    OnTransB( NoDetect );
    OnMatchA(PinHigh);         // set OPAC A according to active-high polarity
    OnMatchB(PinLow);          // set OPAC B according to active-high polarity
    if ( PWMF_ACTIVE_LOW )
    {
      OnMatchA(PinLow);        // set OPAC A according to active-low polarity
      OnMatchB(PinHigh);       // set OPAC B according to active-low polarity
    }
    EnableEventHandling();
    Clear(flag0);
    Pin(PerPacB);              // set pin to inactive state
    ClearAllLatches();
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: STOP                                                         |
| DESCRIPTION: Stop the PWM generation, set pin to inactive state.          |
|              1. Disable matches in thread in case there are any previous  |
|                  pending matches. This could happen if the channel was    |
|                  running another function before this one.                |
|              2. Disable match and transition event handling.              |
|              3. Set pin to inactive state.                                |
+--------------------------------------------------------------------------*/
  else if (hsr == PWMF_HSR_STOP )
  {
    DisableMatchesInThread();
    DisableEventHandling();
    ClearAllLatches();
    if ( PWMF_ACTIVE_LOW )
    {
      Pin(SetHigh);
      OnMatchA(NoChange);
    }
    else
    {
      Pin(SetLow);
      OnMatchA(NoChange);
    }
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: UPDATE_0                                                     |
| DESCRIPTION: Reschedule :                                                 |
|              - in normal upd: Match A (with respect to 0%-duty flag) and  |
|                               Match B (with respect to 100%-duty flag).   |
|              - in half-cycle: Match B (with respect to 100%-duty flag).   |
|              1. If not half-cycle realod                                  |
|                   On base channel:                                        |
|                   - schedule match A at A_time.                           |
|                   - update A_time = A_time + period_copy.                 |
|                   On complementary channel:                               |
|                   - schedule match A at Ac_time.                          |
|                   - update Ac_time = Ac_time + period_copy.               |
|                   If (flag 0% set), set OPAC A = no_change,               |
|                   else set OPAC A according to channel polarity.          |
|              2. On base channel:                                          |
|                 - schedule match B at B_time.                             |
|                 - update B_time = B_time + period_copy.                   |
|                 On complementary channel:                                 |
|                 - schedule match B at Bc_time.                            |
|                 - update Bc_time = Bc_time + period_copy.                 |
|                 If (flag 100% set), set OPAC B = no_change,               |
|                 else set OPAC B according to channel polarity.            |
|              3. Clear link service latch.                                 |
+--------------------------------------------------------------------------*/
  else if ( ((lsr == 1)               && (flag0 == 0)) ||
            ((hsr == PWMF_HSR_UPDATE) && (flag0 == 0)) )
  {
    Clear(lsr);
    if ( !(flags & PWMM_HALF_CYCLE) )
    {
PWMF_AB:
      if ( PWMF_BASE_CHAN )
      {
        erta = A_time;              // Setup Match A, but do not clear mrlA
        EnableMatchA();
        A_time = erta + period_copy;// predict for next period
      }
      else
      {
        erta = Ac_time;             // Setup Match A, but do not clear mrlA
        EnableMatchA();
        Ac_time = erta + period_copy;// predict for next period
      }
      if ( flags & PWMM_ZERO_DUTY )
      {
        OnMatchA(NoChange);        // cancel pin action on matchA
      }
      else
      {
        if ( PWMF_ACTIVE_LOW )
        {
          OnMatchA(PinLow);      // set OPAC A according to active-low polarity
        }
        else
        {
          OnMatchA(PinHigh);     // set OPAC A according to active-high polarity
        }
      }
    }
    goto PWMF_B;
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: UPDATE_1                                                     |
| DESCRIPTION: Reschedule :                                                 |
|              - in normal upd: Match A (with respect to 0%-duty flag).     |
|              - in half-cycle: Match A (with respect to 0%-duty flag) and  |
|                               Match B (with respect to 100%-duty flag).   |
|              1. On base channel:                                          |
|                 - schedule match A at A_time.                             |
|                 - update A_time = A_time + period_copy.                   |
|                 On complementary channel:                                 |
|                 - schedule match A at Ac_time.                            |
|                 - update Ac_time = Ac_time + period_copy.                 |
|                 If (flag 0% set), set OPAC A = no_change,                 |
|                 else set OPAC A according to channel polarity.            |
|              2. If half-cycle reload                                      |
|                   On base channel:                                        |
|                   - schedule match B at B_time.                           |
|                   - update B_time = B_time + period_copy.                 |
|                   On complementary channel:                               |
|                   - schedule match B at Bc_time.                          |
|                   - update Bc_time = Bc_time + period_copy.               |
|                   If (flag 100% set), set OPAC B = no_change,             |
|                   else set OPAC B according to channel polarity.          |
|              3. Clear link service latch.                                 |
+--------------------------------------------------------------------------*/
  else if ( ((lsr == 1)               && (flag0 == 1)) ||
            ((hsr == PWMF_HSR_UPDATE) && (flag0 == 1)) )
  {
    Clear(lsr);
    if ( flags & PWMM_HALF_CYCLE )
      goto PWMF_AB;
    else
      goto PWMF_A;
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: MATCH_A                                                      |
| DESCRIPTION: Schedule match A at on A_time (resp. Ac_time) and            |
|              update A_time (resp. Ac_time) for next period.               |
|              1. On base channel:                                          |
|                 - schedule match A at A_time.                             |
|                 - update A_time = A_time + period_copy.                   |
|                 On complementary channel:                                 |
|                 - schedule match A at Ac_time.                            |
|                 - update Ac_time = Ac_time + period_copy.                 |
|              2. If (flag 0% set), set OPAC A = no_change,                 |
|                 else set OPAC A according to channel polarity.            |
|              3. Set flag0.                                                |
+--------------------------------------------------------------------------*/
  else if ((m1 == 1) && (flag0 == 0))
  {
    ClearMatchALatch();
    Set(flag0);
PWMF_A:
    if ( PWMF_BASE_CHAN )
    {
      erta = A_time;
      EnableMatchA();
      A_time = erta + period_copy;     // predict for next period
    }
    else
    {
      erta = Ac_time;
      EnableMatchA();
      Ac_time = erta + period_copy;    // predict for next period
    }

    if ( flags & PWMM_ZERO_DUTY )
    {
      OnMatchA(NoChange);       // cancel pin action on matchA
    }
    else
    {
      if ( PWMF_ACTIVE_LOW )
      {
        OnMatchA(PinLow);      // set OPAC A according to active-low polarity
      }
      else
      {
        OnMatchA(PinHigh);     // set OPAC A according to active-high polarity
      }
    }
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: MATCH_B                                                      |
| DESCRIPTION: Schedule match B at on B_time (resp. Bc_time) and            |
|              update B_time (resp. Bc_time) for next period.               |
|              1. On base channel:                                          |
|                 - schedule match B at B_time.                             |
|                 - update B_time = B_time + period_copy.                   |
|                 On complementary channel:                                 |
|                 - schedule match B at Bc_time.                            |
|                 - update Bc_time = Bc_time + period_copy.                 |
|              2. If (flag 100% set), set OPAC B = no_change,               |
|                 else set OPAC B according to channel polarity.            |
|              3. Clear flag0.                                              |
+--------------------------------------------------------------------------*/
  else if (m2 == 1 )
  {
    ClearMatchBLatch();
    Clear(flag0);
PWMF_B:
    if ( PWMF_BASE_CHAN )
    {
      ertb = B_time;
      EnableMatchB();
      B_time = ertb + period_copy;     // predict for next period
    }
    else
    {
      ertb = Bc_time;
      EnableMatchB();
      Bc_time = ertb + period_copy;    // predict for next period
    }

    if ( flags & PWMM_FULL_DUTY )
    {
      OnMatchB(NoChange);       // cancel pin action on matchA
    }
    else
    {
      if ( PWMF_ACTIVE_LOW )
      {
        OnMatchB(PinHigh);     // set OPAC B according to active-low polarity
      }
      else
      {
        OnMatchB(PinLow);      // set OPAC A according to active-high polarity
      }
    }
  }
/*--------------------------------------------------------------------------+
| Catch all unspecified entry table conditions                              |
+--------------------------------------------------------------------------*/
  else
  {
      #ifdef SET_USED
      Global_Error_Func();           // Call the Global error routine and end.
      #endif
  }
}

/* Information exported to Host CPU program */
#pragma write h, (::ETPUfilename (cpu/etpu_pwmf_auto.h));
#pragma write h, ( );
#pragma write h, (/****************************************************************/);
#pragma write h, (/* WARNING this file is automatically generated DO NOT EDIT IT! */);
#pragma write h, (/*                                                              */);
#pragma write h, (/* This file provides an interface between eTPU code and CPU    */);
#pragma write h, (/* code. All references to the PWMF function should be made with*/);
#pragma write h, (/* information in this file. This allows only symbolic          */);
#pragma write h, (/* information to be referenced which allows the eTPU code to be*/);
#pragma write h, (/* optimized without effecting the CPU code.                    */);
#pragma write h, (/****************************************************************/);
#pragma write h, (#ifndef _ETPU_PWMF_AUTO_H_ );
#pragma write h, (#define _ETPU_PWMF_AUTO_H_ );
#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Function Configuration Information. );
#pragma write h, (****************************************************************/);
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMF_FUNCTION_NUMBER) PWMF_FUNCTION_NUMBER );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMF_TABLE_SELECT) ::ETPUentrytype(PWMF) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMF_NUM_PARMS) ::ETPUram(PWMF) );
#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Host Service Request Definitions. );
#pragma write h, (****************************************************************/);
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMF_INIT       ) PWMF_HSR_INIT );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMF_UPDATE     ) PWMF_HSR_UPDATE );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMF_STOP       ) PWMF_HSR_STOP );
#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Parameter Definitions. );
#pragma write h, (****************************************************************/);
#pragma write h, (// PWMF function.);
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMF_OPTIONS_OFFSET    ) ::ETPUlocation (PWMF, options) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMF_PERIODCOPY_OFFSET ) ::ETPUlocation (PWMF, period_copy) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMF_DTC_OFFSET        ) ::ETPUlocation (PWMF, dtc) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMF_ATIME_OFFSET      ) ::ETPUlocation (PWMF, A_time) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMF_BTIME_OFFSET      ) ::ETPUlocation (PWMF, Ac_time) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMF_ACTIME_OFFSET     ) ::ETPUlocation (PWMF, B_time) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMF_BCTIME_OFFSET     ) ::ETPUlocation (PWMF, Bc_time) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMF_FLAGS_OFFSET      ) ::ETPUlocation (PWMF, flags) );
#pragma write h, ( );
#pragma write h, (#endif );
#pragma write h, ( );


/*********************************************************************
 *
 * Copyright:
 *	Freescale Semiconductor, INC. All Rights Reserved.
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Freescale
 *  Semiconductor, Inc. This software is provided on an "AS IS"
 *  basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, Freescale
 *  Semiconductor DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED,
 *  INCLUDING IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 *  PARTICULAR PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH
 *  REGARD TO THE SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF)
 *  AND ANY ACCOMPANYING WRITTEN MATERIALS.
 *
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL Freescale Semiconductor BE LIABLE FOR ANY DAMAGES WHATSOEVER
 *  (INCLUDING WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
 *  BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER
 *  PECUNIARY LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.
 *
 *  Freescale Semiconductor assumes no responsibility for the
 *  maintenance and support of this software
 ********************************************************************/
