/*******************************************************************************
*
*  FREESCALE SEMICONDUCTOR INC. 2004
*  ALL RIGHTS RESERVED
*  COPYRIGHT (c)
*  
********************************************************************************
*
*  DESCRIPTION:
*   The Motor-Control PWM - Master (PWMM) function read input parameters,                        
*   calculates duty-cycles and updates the edge-times of up to three PWMC or
*   PWMF phases.                                                          
*
*******************************************************************************
*
*  REVISION HISTORY:
*                                                         
*  REVISION    AUTHOR       DATE        DESCRIPTION OF CHANGE                  
*  --------    -----------  ---------   ------------------------------------                  
*   1.0        M. Brejl     12/Oct/04   HALF_RELOAD implemented. 
*              M. Princ                 PWM modulations not implemented.
*                                       Ready for set3 release.
*
*******************************************************************************/


#ifndef SET_USED
  #pragma entryaddr 0x0000;		          // Base entry table address
  #define PWMM_FUNCTION_NUMBER    1     // Function Number
  #include "etpuc_pwmm.h"               // Shared by PWMM, PWMF and PWMC
#endif

#include <eTPUc.h>          //Defines eTPU hardware
#include <eTPUc_common.h>   //Standard way to use eTPU

#pragma ETPU_function PWMM, alternate @ PWMM_FUNCTION_NUMBER;

//************************************************************************
//*  Definitions.
//*************************************************************************/

// Host service requests
#define PWMM_HSR_INIT_NORMAL       7
#define PWMM_HSR_INIT_HALF_CYCLE   5
#define PWMM_HSR_UPDATE            2

// Function Modes
#define PWMM_EDGE_ALIGNED          (fm0==0)
#define PWMM_CENTER_ALIGNED        (fm0==1)
#define PWMM_SINGLE_CHANNELS       (fm1==0)
#define PWMM_COMPLEMENTARY_PAIRS   (fm1==1)

// Modulations
#define PWMM_MODULATION_NO         0
#define PWMM_MODULATION_UNSIGNED   1
#define PWMM_MODULATION_SIGNED     2
#define PWMM_MODULATION_SVMSTD     3
#define PWMM_MODULATION_SVMU0N     4
#define PWMM_MODULATION_SVMU7N     5
#define PWMM_MODULATION_TABLE      6

// Channel Flag usage
// Flag0 is used to indentify that the update happend. The reload can then
//   process the updated duty-cycles.
// Flag1 - reserved.

/*--------------------------------------------------------------------------+
| FUNCTION PARAMETERS:                                                      |
| phases: Number of used phases.                                            |
| modulation: Selection of modulation. In case of "PWMM_MODULATION_NO" the  |
|             phase duty cycles are the inputs, otherwise the phase duty    |
|             cycles are calculated based on the inputs.                    |
| start_offset: This parameter is used to synchronize various eTPU functions|
|               that generate a signal. For PWMM: the first PWM period start|
|               start_offset TCR1 clocks after initialization.              |
| update_time: Time (as number of TCR1 clocks) that is necessary to perform |
|              the update of all PWM phases.                                |
| dead_time: Dead-time as number of TCR1 clocks.                            |
| min_pw: Minimum pulse width as number of TCR1 clocks                      |
| period: PWM period. Can be updated each UPDATE.                           |
| inputs: Depending on the selected modulation:                             |
|         .A - duty cycle value            .B -                             |
|            - alpha (SVMs)                   - beta (SVMs)                 |
|            - amplitude (sine modulations    - angle (sine modulations)    |
| table: Pointer to the first quadrant table. Applies only if Table         |
|        Modulation is selected. The table is an array of unsigned 16-bit   |
|        fract values, and of length 64.                                    |
| sector: In case of an SVM modulatin the Applied Voltage Vector position   |
|         in one of 6 sectors is returned.                                  |
| edge_time: Next PWM period edge-time. Updated each period.                |
| chnum_duty[3]: Array of all phases channel number (upper 8 bits) and      |
|                duty-cycle (lower 24 bits). If modulation == 0, then       |
|                the duty-cycle is enetered by CPU, otherwise resulting     |
|                from the modulation algorithm                              |
|                                                                           |
| STATIC LOCAL VARIABLES:                                                   |
| period_copy: Local copy of the period parameter.                          |
+--------------------------------------------------------------------------*/
void PWMM (             int8  phases,
                       int24  start_offset,
                        int8  modulation,
                       int24  update_time,
                       int24  dead_time,
                       int24  min_pw,
                       int24  period,
               pwmm_inputs_t  inputs,
            unsigned fract16 *table,
                       int8   sector,
                       int24  edge_time,
           pwmm_chnum_duty_t  chnum_dutyA,
           pwmm_chnum_duty_t  chnum_dutyB,
           pwmm_chnum_duty_t  chnum_dutyC,
                pwmm_phase_t  phaseA,
                pwmm_phase_t  phaseB,
                pwmm_phase_t  phaseC)

                       

{
  static int24 period_copy;
         int24 tmp_period;
         int24 tmp_max_pw;
         int24 tmp_high_time;

/*--------------------------------------------------------------------------+
| THREAD NAME: INIT_NORMAL                                                  |
| DESCRIPTION: Initialize a channel to run the PWMM function with normal    |
|              reload.                                                      |
|              1. Disable matches in thread in case there are any previous  |
|                  pending matches. This could happen if the channel was    |
|                  running another function before this one.                |
|              2. Select SingleMatch-SingleTransition channel mode.         |
|              3. Configure channel to use TCR1 clock.                      |
|              4. Configure input pin actions to no_detect.                 |
|              5. Configure output pin actions to no_change.                |
|              6. Enable match and transition event handling.               |
|              7. Clear flag0.                                              |
|              8. Set pin low.                                              |
|              9. Clear latches.                                            |
|             10. Send link to self to issue the first update.              |
|             11. Set edge_time = tcr1 + start_offset.                      |
|             12. Schedule matchA at edge-time - update time.               |
+--------------------------------------------------------------------------*/
  if (hsr == PWMM_HSR_INIT_NORMAL)
  {
    DisableMatchesInThread();
    SingleMatchSingleTransition();
    ActionUnitA( MatchTCR1, CaptureTCR1, GreaterEqual);
    EnableOutputBuffer();
    OnTransA(NoDetect);
    OnMatchA(NoChange);
    EnableEventHandling();
    Clear(flag0);
    Pin(SetLow);
    ClearAllLatches();

    LinkTo(GetCurrentChanNum());
    edge_time = tcr1 + start_offset;
    SetupMatchA(edge_time - update_time);
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: INIT_HALF_CYCLE                                              |
| DESCRIPTION: Initialize a channel to run the PWMM function with half-cycle|
|              reload.                                                      |
|              1. Disable matches in thread in case there are any previous  |
|                  pending matches. This could happen if the channel was    |
|                  running another function before this one.                |
|              2. Select EitherMatch-Blocking-SingleTransition channel mode.|
|              3. Configure channel to use TCR1 clock.                      |
|              4. Configure input pin actions to no_detect.                 |
|              5. Configure output pin actions to no_change.                |
|              6. Enable match and transition event handling.               |
|              7. Clear flag0.                                              |
|              8. Set pin low.                                              |
|              9. Clear latches.                                            |
|             10. Send link to self to issue the first update.              |
|             11. Set edge_time = tcr1 + start_offset.                      |
|             12. Schedule matchA at edge-time - update time.               |
+--------------------------------------------------------------------------*/
  else if (hsr == PWMM_HSR_INIT_HALF_CYCLE)
  {
    DisableMatchesInThread();
    EitherMatchNonBlockingSingleTransition();
    ActionUnitA( MatchTCR1, CaptureTCR1, GreaterEqual);
    ActionUnitB( MatchTCR1, CaptureTCR1, GreaterEqual);
    OnTransA(NoDetect);
    OnTransB(NoDetect);
    OnMatchA(NoChange);
    OnMatchB(NoChange);
    EnableEventHandling();
    Clear(flag0);
    Pin(SetLow);
    ClearAllLatches();

    LinkTo(GetCurrentChanNum());
    edge_time = tcr1 + start_offset;
    SetupMatchA(edge_time - update_time);
    SetupMatchB(erta + (period>>1));
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: UPDATE                                                       |
| DESCRIPTION: Read new input values, perform the modulation and update the |
|              phase duty-cycles.                                           |
|              1. Set pin high to indicate the calculation time.            |
|              2. Based on the modulation parameter calculate duty-cycles.  |
|              3. Set flag0 to indicate that the update happend.            |
|              4. Set pin low.                                              |
+--------------------------------------------------------------------------*/
  else if ( ((lsr==1) && (m1==0) && (m2==0)) || (hsr == PWMM_HSR_UPDATE) )
  {
    Pin(SetHigh);
    Set(flag0);
    switch (modulation)
    {
      case PWMM_MODULATION_UNSIGNED:
        tmp_period = inputs.A;
        chnum_dutyA.duty = tmp_period;
        chnum_dutyB.duty = tmp_period;
        chnum_dutyC.duty = tmp_period;
        break;
      case PWMM_MODULATION_SIGNED:
        tmp_period = inputs.A + 0x800000;
        chnum_dutyA.duty = tmp_period;
        chnum_dutyB.duty = tmp_period;
        chnum_dutyC.duty = tmp_period;
        break;
      case PWMM_MODULATION_TABLE:
        break;
      case PWMM_MODULATION_SVMSTD:
        break;
      case PWMM_MODULATION_SVMU0N:
        break;
      case PWMM_MODULATION_SVMU7N:
    }

    Clear(lsr);
    Pin(SetLow);
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: RELOAD                                                       |
| DESCRIPTION: Calculate and set all edge-times of each running phase based |
|              on the phase duty-cycles.                                    |
|              1. Set pin high to indicate the calculation time.            |
|              2. Save period to period_copy                                |
|              3. for each running phase X ( X = A, B, C )                  |
|               - save period_copy to channelX period_copy                  |
|               - calculate high-time, consider channelX option"negate duty"|
|               - adjust high-time using dtcX                               |
|               - limit high-time using min_pw, set channelX duty_extreme   |
|                 flags                                                     |
|               - calculate channel X A_time, B_time, Ac_time, Bc_time,     |
|                 consider options "edge X center-aligned" and channelX     |
|                 option "swap dead-time insertion"                         |
|               - clear channelX half-cycle reload flag                     |
|               - send link to channelX and if complementary pairs to X+1   |
|              4. Clear flag0.                                              |
|              5. Continue with RELOAD_MISSED                               |
+--------------------------------------------------------------------------*/
  else if ( (m1==1) && (flag0==1) )
  {
    DisableMatchesInThread();
    Pin(SetHigh);
    Clear(flag0);
    
RELOAD:
    tmp_period = period;
    //tmp_max_pw = tmp_period - (dead_time<<1) - min_pw;   compiler bug
    tmp_max_pw = tmp_period - min_pw - (dead_time<<1);
    
    switch (phases)
    {
     case 3:
       //***************** Phase C *************************
       // send link to channelX and if complementary pairs to X+1
       LinkTo(chnum_dutyC.chnum);
       if (PWMM_COMPLEMENTARY_PAIRS)
       {
         LinkTo(chnum_dutyC.chnum+1);
       }
       // save period_copy to channelX period_copy
       phaseC.period_copy = tmp_period;
         phaseC.flags = 0;
       // calculate high-time, consider channelX option "negate duty"
       tmp_high_time = tmp_period * chnum_dutyC.duty;
       if ( phaseC.options & PWMM_DUTY_NEG )
       {
         tmp_high_time = tmp_period - tmp_high_time;
       }
       tmp_high_time -= dead_time;   // no dead-time compensation for now
       // limit high-time using min_pw, set channelX duty_extreme flags 
       if ( tmp_high_time < min_pw )
       {
         if ( (tmp_high_time - (min_pw>>1)) < 0 )  // tmp_high_time < (min_pw>>1) 
           phaseC.flags = PWMM_ZERO_DUTY;
         tmp_high_time = min_pw;
       }
       else if ( tmp_high_time > tmp_max_pw )
       {
         if ( tmp_high_time > (tmp_max_pw + min_pw>>1))
           phaseC.flags = PWMM_FULL_DUTY;
         tmp_high_time = tmp_max_pw;
       }
       // calculate channel X A_time, B_time, Ac_time, Bc_time, 
       // consider options "edge X center-aligned" and channelX option "swap dead-time insertion".
       erta = edge_time;   // time of first edge in case of edge-aligned
       if ( PWMM_CENTER_ALIGNED )
         erta += (tmp_period - tmp_high_time)>>1;  // in case of center-aligned
       if ( IsMatchALatched() )
         ertb = erta + tmp_high_time;              // in case of RELOAD thread
       else
         ertb = erta - tmp_period + tmp_high_time; // in case of HALF_RELOAD thread

       // save A/B/Ac/Bc_times
       if (phaseC.options & PWMM_SWAP) {
         phaseC.Ac_time = erta;
         phaseC.Bc_time = ertb;
         phaseC.A_time = erta - dead_time;
         phaseC.B_time = ertb + dead_time;
       }
       else {
         phaseC.A_time = erta;
         phaseC.B_time = ertb;
         phaseC.Ac_time = erta - dead_time;
         phaseC.Bc_time = ertb + dead_time;
       }
       // adjust channelX flags
       if ( IsMatchALatched() )
         phaseC.flags &= ~PWMM_HALF_CYCLE;    // in case of RELOAD thread     
       else                                                               
         phaseC.flags |= PWMM_HALF_CYCLE;     // in case of HALF_RELOAD thread

     case 2:
       //***************** Phase B *************************
       // send link to channelX and if complementary pairs to X+1
       LinkTo(chnum_dutyB.chnum);
       if (PWMM_COMPLEMENTARY_PAIRS)
       {
         LinkTo(chnum_dutyB.chnum+1);
       }
       // save period_copy to channelX period_copy
       phaseB.period_copy = tmp_period;
         phaseB.flags = 0;
       // calculate high-time, consider channelX option "negate duty"
       tmp_high_time = tmp_period * chnum_dutyB.duty;
       if ( phaseB.options & PWMM_DUTY_NEG )
       {
         tmp_high_time = tmp_period - tmp_high_time;
       }
       tmp_high_time -= dead_time;   // no dead-time compensation for now
       // limit high-time using min_pw, set channelX duty_extreme flags 
       if ( tmp_high_time < min_pw )
       {
         if ( (tmp_high_time - (min_pw>>1)) < 0 )  // tmp_high_time < (min_pw>>1) 
           phaseB.flags = PWMM_ZERO_DUTY;
         tmp_high_time = min_pw;
       }
       else if ( tmp_high_time > tmp_max_pw )
       {
         if ( tmp_high_time > (tmp_max_pw + min_pw>>1))
           phaseB.flags = PWMM_FULL_DUTY;
         tmp_high_time = tmp_max_pw;
       }
       // calculate channel X A_time, B_time, Ac_time, Bc_time, 
       // consider options "edge X center-aligned" and channelX option "swap dead-time insertion".
       erta = edge_time;   // time of first edge in case of edge-aligned
       if ( PWMM_CENTER_ALIGNED )
         erta += (tmp_period - tmp_high_time)>>1;  // in case of center-aligned
       if ( IsMatchALatched() )
         ertb = erta + tmp_high_time;              // in case of RELOAD thread     
       else                                                                        
         ertb = erta - tmp_period + tmp_high_time; // in case of HALF_RELOAD thread

       // save A/B/Ac/Bc_times
       if (phaseB.options & PWMM_SWAP) {
         phaseB.Ac_time = erta;
         phaseB.Bc_time = ertb;
         phaseB.A_time = erta - dead_time;
         phaseB.B_time = ertb + dead_time;
       }
       else {
         phaseB.A_time = erta;
         phaseB.B_time = ertb;
         phaseB.Ac_time = erta - dead_time;
         phaseB.Bc_time = ertb + dead_time;
       }
       // adjust channelX flags
       if ( IsMatchALatched() )
         phaseB.flags &= ~PWMM_HALF_CYCLE;    // in case of RELOAD thread     
       else                                                                   
         phaseB.flags |= PWMM_HALF_CYCLE;     // in case of HALF_RELOAD thread

     case 1:
        //***************** Phase A *************************
       // send link to channelX and if complementary pairs to X+1
       LinkTo(chnum_dutyA.chnum);
       if (PWMM_COMPLEMENTARY_PAIRS)
       {
         LinkTo(chnum_dutyA.chnum+1);
       }
       // save period_copy to channelX period_copy
       phaseA.period_copy = tmp_period;
         phaseA.flags = 0;
       // calculate high-time, consider channelX option "negate duty"
       tmp_high_time = tmp_period * chnum_dutyA.duty;
       if ( phaseA.options & PWMM_DUTY_NEG )
       {
         tmp_high_time = tmp_period - tmp_high_time;
       }
       tmp_high_time -= dead_time;   // no dead-time compensation for now
       // limit high-time using min_pw, set channelX duty_extreme flags 
       if ( tmp_high_time < min_pw )
       {
         if ( (tmp_high_time - (min_pw>>1)) < 0 )
           phaseA.flags = PWMM_ZERO_DUTY;
         tmp_high_time = min_pw;
       }
        else if ( tmp_high_time > tmp_max_pw )
       {
         if ( tmp_high_time > (tmp_max_pw + min_pw>>1))
           phaseA.flags = PWMM_FULL_DUTY;
         tmp_high_time = tmp_max_pw;
       }
       // calculate channel X A_time, B_time, Ac_time, Bc_time, 
       // consider options "edge X center-aligned" and channelX option "swap dead-time insertion".
       erta = edge_time;   // time of first edge in case of edge-aligned
       if ( PWMM_CENTER_ALIGNED )
         erta += (tmp_period - tmp_high_time)>>1;  // in case of center-aligned
       if ( IsMatchALatched() )
         ertb = erta + tmp_high_time;              // in case of RELOAD thread     
       else                                                                        
         ertb = erta - tmp_period + tmp_high_time; // in case of HALF_RELOAD thread

       // save A/B/Ac/Bc_times
       if (phaseA.options & PWMM_SWAP) {
         phaseA.Ac_time = erta;
         phaseA.Bc_time = ertb;
         phaseA.A_time = erta - dead_time;
         phaseA.B_time = ertb + dead_time;
       }
       else {
         phaseA.A_time = erta;
         phaseA.B_time = ertb;
         phaseA.Ac_time = erta - dead_time;
         phaseA.Bc_time = ertb + dead_time;
       }
       // adjust channelX flags
       if ( IsMatchALatched() )
         phaseA.flags &= ~PWMM_HALF_CYCLE;    // in case of RELOAD thread     
       else                                                                   
         phaseA.flags |= PWMM_HALF_CYCLE;     // in case of HALF_RELOAD thread
       //***************** Phases done *****************************************
    
    }
    period_copy = tmp_period;
    if ( IsMatchALatched() )
      goto PWMM_RELOAD_COMMON;
    else
      goto PWMM_HALF_RELOAD_COMMON;
 
  }
    
/*--------------------------------------------------------------------------+
| THREAD NAME: RELOAD_MISSED                                                |
| DESCRIPTION: Do not set any edge-times (phases continue with the same     |
|              duty-cycle), do not update period_copy by period.            |
|              Update edge_time and schedule next RELOAD.                   |
|              1. Set pin high to indicate the calculation time.            |
|              2. Update edge_time by period_copy.                          |
|              3. Schedule matchA at edge_time - update_time.               |
|              4. Negate mrlA.                                              |
|              5. Set pin low.                                              |
+--------------------------------------------------------------------------*/
  else if ( (m1 == 1) && (flag0 == 0) )
  {
    Pin(SetHigh);
PWMM_RELOAD_COMMON:
    edge_time += period_copy;
    SetupMatchA(edge_time - update_time);
    Pin(SetLow);
  }
/*--------------------------------------------------------------------------+
| THREAD NAME: HALF_RELOAD                                                  |
| DESCRIPTION: Calculate and set all edge-times of each running phase based |
|              on the phase duty-cycles.                                    |
|              1. Set pin high to indicate the calculation time.            |
|              2. for each running phase X ( X = A, B, C )                  |
|               - calculate high-time, consider channelX option"negate duty"|
|               - adjust high-time using dtcX                               |
|               - limit high-time using min_pw, set channelX duty_extreme   |
|                 flags                                                     |
|               - calculate channel X A_time, B_time, Ac_time, Bc_time,     |
|                 consider options "edge X center-aligned" and channelX     |
|                 option "swap dead-time insertion"                         |
|               - set channelX half-cycle reload flag                       |
|               - send link to channelX and if complementary pairs to X+1   |
|              3. Clear flag0.                                              |
|              4. Continue with HALF_RELOAD_MISSED                          |
+--------------------------------------------------------------------------*/
  else if ( (m2 == 1) && (flag0 == 1) )
  {
    DisableMatchesInThread();
    Pin(SetHigh);
    Clear(flag0);
 
    goto RELOAD;
  } 
/*--------------------------------------------------------------------------+
| THREAD NAME: HALF_RELOAD_MISSED                                           |
| DESCRIPTION: Do not set any edge-times (phases continue with the same     |
|              duty-cycle).                                                 |
|              Schedule next HALF_RELOAD.                                   |
|              1. Set pin high to indicate the calculation time.            |
|              2. Schedule matchB at edge_time + period_copy/2 - update_time|
|              3. Negate mrlB.                                              |
|              4. Set pin low.                                              |
+--------------------------------------------------------------------------*/
  else if ( (m2 == 1) && (flag0 == 0) )
  {
    Pin(SetHigh);
PWMM_HALF_RELOAD_COMMON:
    SetupMatchB(edge_time + (period>>1) - update_time);
    Pin(SetLow);
  }
/*--------------------------------------------------------------------------+
| Catch all unspecified entry table conditions                              |
+--------------------------------------------------------------------------*/
  else
  {
      #ifdef SET_USED
      Global_Error_Func();           // Call the Global error routine and end.
      #endif
  }
}

/* Information exported to Host CPU program */
#pragma write h, (::ETPUfilename (cpu/etpu_pwmm_auto.h));
#pragma write h, ( );
#pragma write h, (/****************************************************************/);
#pragma write h, (/* WARNING this file is automatically generated DO NOT EDIT IT! */);
#pragma write h, (/*                                                              */);
#pragma write h, (/* This file provides an interface between eTPU code and CPU    */);
#pragma write h, (/* code. All references to the PWMM function should be made with*/);
#pragma write h, (/* information in this file. This allows only symbolic          */);
#pragma write h, (/* information to be referenced which allows the eTPU code to be*/);
#pragma write h, (/* optimized without effecting the CPU code.                    */);
#pragma write h, (/****************************************************************/);
#pragma write h, (#ifndef _ETPU_PWMM_AUTO_H_ );
#pragma write h, (#define _ETPU_PWMM_AUTO_H_ );
#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Function Configuration Information. );
#pragma write h, (****************************************************************/);
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_FUNCTION_NUMBER) PWMM_FUNCTION_NUMBER );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_TABLE_SELECT) ::ETPUentrytype(PWMM) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_NUM_PARMS) ::ETPUram(PWMM) );
#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Host Service Request Definitions. );
#pragma write h, (****************************************************************/);
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_INIT_NORMAL     ) PWMM_HSR_INIT_NORMAL     );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_INIT_HALF_CYCLE ) PWMM_HSR_INIT_HALF_CYCLE );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_UPDATE          ) PWMM_HSR_UPDATE          );
#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Parameter Definitions. );
#pragma write h, (****************************************************************/);
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_PHASES_OFFSET       ) ::ETPUlocation (PWMM, phases      ) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_STARTOFFSET_OFFSET  ) ::ETPUlocation (PWMM, start_offset) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_MODULATION_OFFSET   ) ::ETPUlocation (PWMM, modulation  ) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_UPDATETIME_OFFSET   ) ::ETPUlocation (PWMM, update_time ) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_DEADTIME_OFFSET     ) ::ETPUlocation (PWMM, dead_time   ) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_MINPW_OFFSET        ) ::ETPUlocation (PWMM, min_pw      ) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_PERIOD_OFFSET       ) ::ETPUlocation (PWMM, period      ) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_INPUTS_OFFSET       ) ::ETPUlocation (PWMM, inputs      ) );
//#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_TABLE_OFFSET        ) ::ETPUlocation (PWMM, table       ) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_SECTOR_OFFSET       ) ::ETPUlocation (PWMM, sector      ) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_EDGETIME_OFFSET     ) ::ETPUlocation (PWMM, edge_time   ) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_CHNUMDUTYA_OFFSET   ) ::ETPUlocation (PWMM, chnum_dutyA ) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_CHNUMDUTYB_OFFSET   ) ::ETPUlocation (PWMM, chnum_dutyB ) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_CHNUMDUTYC_OFFSET   ) ::ETPUlocation (PWMM, chnum_dutyC ) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_PHASEA_OFFSET       ) ::ETPUlocation (PWMM, phaseA      ) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_PHASEB_OFFSET       ) ::ETPUlocation (PWMM, phaseB      ) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_PHASEC_OFFSET       ) ::ETPUlocation (PWMM, phaseC      ) );

#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Value Definitions. );
#pragma write h, (****************************************************************/);
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_MODULATION_NO       ) PWMM_MODULATION_NO       );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_MODULATION_UNSIGNED ) PWMM_MODULATION_UNSIGNED );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_MODULATION_SIGNED   ) PWMM_MODULATION_SIGNED   );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_MODULATION_SVMSTD   ) PWMM_MODULATION_SVMSTD   );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_MODULATION_SVMU0N   ) PWMM_MODULATION_SVMU0N   );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_MODULATION_SVMU7N   ) PWMM_MODULATION_SVMU7N   );
#pragma write h, (::ETPUliteral(#define FS_ETPU_PWMM_MODULATION_TABLE    ) PWMM_MODULATION_TABLE    );
#pragma write h, ( );
#pragma write h, (#endif );
#pragma write h, ( );


/*********************************************************************
 *
 * Copyright:
 *	Freescale Semiconductor, INC. All Rights Reserved.
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Freescale
 *  Semiconductor, Inc. This software is provided on an "AS IS"
 *  basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, Freescale
 *  Semiconductor DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED,
 *  INCLUDING IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 *  PARTICULAR PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH
 *  REGARD TO THE SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF)
 *  AND ANY ACCOMPANYING WRITTEN MATERIALS.
 *
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL Freescale Semiconductor BE LIABLE FOR ANY DAMAGES WHATSOEVER
 *  (INCLUDING WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
 *  BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER
 *  PECUNIARY LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.
 *
 *  Freescale Semiconductor assumes no responsibility for the
 *  maintenance and support of this software
 ********************************************************************/
