/*******************************************************************************
*
*  FREESCALE SEMICONDUCTOR INC. 2004
*  ALL RIGHTS RESERVED
*  COPYRIGHT (c)
*  
********************************************************************************
*
*  DESCRIPTION:
*   Speed Controller eTPU function
*
********************************************************************************
*
*  REVISION HISTORY:
*                                                         
*  REVISION    AUTHOR       DATE        DESCRIPTION OF CHANGE                  
*  --------    -----------  ---------   ------------------------------------                  
*  1.0         M. Princ     12/Oct/04    PID code updated with new version
*                                        of compiler, ready for release.
*
*******************************************************************************/

#ifndef SET_USED
  #pragma entryaddr 0x0000;           // Base entry table address
  #define SC_FUNCTION_NUMBER      0   // Function number.
  #include "etpuc_mc.h"
#endif

#include <eTPUc.h>                    //Defines eTPU hardware
#include <eTPUc_common.h>             //Standard way to use eTPU

#pragma ETPU_function SC, standard @ SC_FUNCTION_NUMBER;

/*******************************************************************************
*  Definitions. 
*******************************************************************************/

// HSR values.
#define SC_HSR_INIT_HD_MASTER      7
#define SC_HSR_INIT_HD_SLAVE       6
#define SC_HSR_INIT_QD_MASTER      5
#define SC_HSR_INIT_QD_SLAVE       4
#define SC_HSR_CALC                1

// FM modes.
#define SC_PID_OFF                 (fm0==0)
#define SC_PID_ON                  (fm0==1)
#define SC_RAMP_OFF                (fm1==0)
#define SC_RAMP_ON                 (fm1==1)

// Options.
#define SC_P_SCALE                 0x01
#define SC_I_SCALE                 0x02
#define SC_D_SCALE                 0x04

// flags
#define SC_SATUR_POS_FLAG          0x01
#define SC_SATUR_NEG_FLAG          0x02

/*******************************************************************************
* Channel Flag usage:
* Flag0 is used to select speed calculation based on HD or QD.
*    0 = Calculate speed from HD parameters
*    1 = Calculate speed from QD parameters
*******************************************************************************/

/* the SC function accesses the mach register 
   in order to get the reminder after division       */
     register_mach      mach ;      // 24 bits

/*******************************************************************************
* NAME: SC
*
* DESCRIPTION: Speed Controller. 
* 
* FUNCTION PARAMETERS:
*  int8      options              - I, D on/off
*  int24     period               - controller period (applies in master
*                                   mode only)
*  int24     start_offset         - start delay (in TCR1 ticks)
*  int24     services_per_irq     - number of periods per one CPU IRQ
*  int24     omega_actual         - actual omega value
*  int24     omega_desired        - desired omega value
*  int24     omega_desired_ramp   - ramp output
*  int24   * p_hd_period          - pointer to HD sector_period or 
*                                   revolution_period parameter
*  int24   * p_hd_direction       - pointer to HD direction parameter
*  int24     max_hd_period        - maximum value of HD period.
*  int24   * p_qd_pc              - pointer to QD position counter
*  int24   * p_qd_tcr_value       - pointer to TCR time of last QD transition
*  int24     qd_last_pc           - last QD position counter value
*  int24     qd_last_tcr          - last TCR time of last QD transition
*  int24   * p_output             - pointer to PID output receiver
*  int8      link_chan            - channel to send link to
*  int24     scale                - scale for speed calculation
*  int24     ramp_incr_up         - ramp parameter: increment up
*  int24     ramp_incr_down       - ramp parameter: increment down
*  int8      flags                - controller limitation flags
*  uint24    P_gain               - proportional gain
*  uint24    I_gain               - integral gain
*  uint24    D_gain               - derivative gain
*  int24     I_k1                 - PID integral portion
*  int24     limit_positive       - positive controller limit (16bit)
*  int24     limit_negative       - negative controller limit (16bit)
*  int24     output               - PID output value
*  int24     error                - PID actual error value
*  int24     error_k1             - PID error value in previous step
*******************************************************************************/
void SC(  int8      options,
          int24     period,
          int24     start_offset,
 unsigned int24     services_per_irq,
          int24     omega_actual,
          int24     omega_desired,
          int24     omega_desired_ramp,
          int24   * p_hd_period,
          int24   * p_hd_direction,
          int24     max_hd_period,
          int24   * p_qd_pc, 
          int24   * p_qd_tcr_value, 
          int24     qd_last_pc, 
          int24     qd_last_tcr,
          int24   * p_output,
          int8      link_chan,
          int24     scale,
          int24     ramp_incr_up,
          int24     ramp_incr_down,
          int8      flags,
          fract24   P_gain,
          fract24   I_gain,
          fract24   D_gain,
          int24     I_k1,
          int24     limit_positive,
          int24     limit_negative,
          int24     output,
          int24     error,
          int24     error_k1)
{
   static int24 edge_time;                 // Time of first edge. Updated each period.
   static unsigned int24 service_counter;  // Counts down from services_per_irq to zero.
   static int24 qd_position_difference;
   static int24 qd_time_difference;

          int24 portion;
          int24 tmp;

/*******************************************************************************
* THREAD NAME: INIT_HD_MASTER
* DESCRIPTION: Initialize a channel to run the SC function in master mode,
*              with speed calculation based on HD parameters.
*              1. Disable matches in thread in case there are any previous
*                 pending matches. This could happen if the channel was
*                 running another function before this one.
*              2. Clear flag0 - speed calculation based on HD parameters.
*              3. Configure pin actions to no_detect on transition and
*                 no_change on match.
*              4. Configure channel to use TCR1 clock.
*              5. Select SingleMatch-SingleTransition channel mode.
*              6. Enable match and transition event handling.
*              7. Schedule first match after start_offset.
*              8. Set pin low.
*              9. Clear latches.
*             10. Reset service counter.
*******************************************************************************/
   if (hsr==SC_HSR_INIT_HD_MASTER)
   {
      DisableMatchesInThread();

      Clear(flag0);
SC_HSR_INIT_MASTER_COMMON:
      OnTransA(NoDetect);
      OnMatchA(NoChange );
      ActionUnitA(MatchTCR1, CaptureTCR1, GreaterEqual);
      SingleMatchSingleTransition();
      EnableEventHandling(); 

      erta = tcr1+start_offset;      // Time of first match.
      edge_time = erta;              // Save time of first match to edge_time.
      EnableMatchA();                // Schedule the match

SC_HSR_INIT_COMMON:
      SetPinLow();
      EnableOutputBuffer();
      ClearAllLatches();
      service_counter = services_per_irq;
   }
/*******************************************************************************
* THREAD NAME: INIT_HD_SLAVE
* DESCRIPTION: Initialize a channel to run the SC function in slave mode,
*              with speed calculation based on HD parameters.
*              1. Disable matches in thread in case there are any previous
*                 pending matches. This could happen if the channel was
*                 running another function before this one.
*              2. Clear flag0 - speed calculation based on HD parameters.
*              3. Set pin low.
*              4. Clear latches.
*              5. Reset service counter.
*******************************************************************************/
   else if (hsr==SC_HSR_INIT_HD_SLAVE)
   {
      DisableMatchesInThread();

      Clear(flag0);

      goto SC_HSR_INIT_COMMON;
   }
/*******************************************************************************
* THREAD NAME: INIT_QD_MASTER
* DESCRIPTION: Initialize a channel to run the SC function in master mode,
*              with speed calculation based on HD parameters.
*              1. Disable matches in thread in case there are any previous
*                 pending matches. This could happen if the channel was
*                 running another function before this one.
*              2. Set flag0 - speed calculation based on QD parameters.
*              3. Initialize last_pc and last_tcr.
*              4. Configure pin actions to no_detect on transition and
*                 no_change on match.
*              5. Configure channel to use TCR1 clock.
*              6. Select SingleMatch-SingleTransition channel mode.
*              7. Enable match and transition event handling.
*              8. Schedule first match after start_offset.
*              9. Set pin low.
*             10. Clear latches.
*             11. Reset service counter.
*******************************************************************************/
   else if (hsr==SC_HSR_INIT_QD_MASTER)
   {
      DisableMatchesInThread();

      Set(flag0);

      qd_last_pc=*p_qd_pc;
      qd_last_tcr=*p_qd_tcr_value;

      goto SC_HSR_INIT_MASTER_COMMON;
   }
/*******************************************************************************
* THREAD NAME: INIT_QD_SLAVE
* DESCRIPTION: Initialize a channel to run the SC function in slave mode,
*              with speed calculation based on QD parameters.
*              1. Disable matches in thread in case there are any previous
*                 pending matches. This could happen if the channel was
*                 running another function before this one.
*              2. Set flag0 - speed calculation based on QD parameters.
*              3. Initialize last_pc and last_tcr.
*              4. Set pin low.
*              5. Clear latches.
*              6. Reset service counter.
*******************************************************************************/
   else if (hsr==SC_HSR_INIT_QD_SLAVE)
   {
      DisableMatchesInThread();

      Set(flag0);

      qd_last_pc=*p_qd_pc;
      qd_last_tcr=*p_qd_tcr_value;

      goto SC_HSR_INIT_COMMON;
   }
/*******************************************************************************
* THREAD NAME: CALC_HD
* DESCRIPTION: Perform all SC calculations. The speed calculation is based on 
*              HD parameters.
*              1. Set pin high.
*              2. Actual speed calculation based on HD parameters.
*              3. Adjustment of desired speed to speed from defined ramp.
*              4. PID Controller calculations.
*              5. Generate Interrupt each services_per_irq period.
*              6. Send link to link_chan.
*              7. Clear link service latch.
*              8. Set pin low.
*******************************************************************************/
   else if (((link==1) && (flag0==0)) || 
            ((hsr==SC_HSR_CALC) && (flag0==0)))
   {
SC_CALC_HD:
      Pin(SetHigh);
    
      //**********************************************
      //* Speed Calculation.
      //* omega_actual=(0x7FFFFF/(*p_hd_period))*scale
      //**********************************************
      portion = *p_hd_period;
      if (portion > max_hd_period)
      {
         omega_actual = 0;
      }
      else
      {
         portion = 0x7FFFFF/portion;
         tmp = scale;
         if (*p_hd_direction != 0)
         {
            tmp = -tmp;
         }
         omega_actual = portion * tmp;
      }
SC_CALC_COMMON:
      //**********************************************
      //* RAMP.
      //**********************************************
      /* if there is a space to increment, increment up   */
      if ((SC_RAMP_ON) && (omega_desired_ramp <= (omega_desired - ramp_incr_up)))
      {
         omega_desired_ramp = omega_desired_ramp + ramp_incr_up;
      }
      /* if there is a space to decrement, decrement down */   
      else if ((SC_RAMP_ON) && (omega_desired_ramp > (omega_desired + ramp_incr_down)))
      {
         omega_desired_ramp = omega_desired_ramp - ramp_incr_down;
      }
      else
      {
         omega_desired_ramp = omega_desired;
      }

      //**********************************************
      //* PID Controller.
      //**********************************************
      if(SC_PID_ON) 
      {
         /* 1) Calculate the input error value with saturation (MIN24, MAX24)*/
         error = omega_desired_ramp - omega_actual;
         SC_1: /* label used to start new instruction (prevent optimization) */
         if (CC.V == 1)
         {
            if (CC.N == 1)
            {
               error = MAX24;
            }
            else
            {
               error = MIN24;
            }
         }
   
         /* 2) Calculate the proportional portion - always      */
         tmp = error * P_gain;
   
         /* 3) Calculate the integral portion                   */
         if (options & SC_I_SCALE) 
         {
             portion = (error * I_gain);
             portion += I_k1;
             /* limit the integral portion */
             if (portion >= limit_positive) 
             { 
                 portion = limit_positive; 
             }
             else if (portion < limit_negative)
             {
                 portion = limit_negative; 
             }
             /* save the integral portion for next calculation */
             I_k1 = portion;
         
             /* add integral portion to the output */
             tmp += portion; // P_portion + I_portion
         }

         /* 4) Calculate the derivative portion                 */
         if (options & SC_D_SCALE)
         {
             portion = (error - error_k1);
             SC_2: /* label used to start new instruction (prevent optimization) */
             if (CC.V == 1)
             {
                if (CC.N == 1)
                {
                   portion = MAX24;
                }
                else
                {
                   portion = MIN24;
                }
             }
             portion *= D_gain;

             /* save error for next calculation */
             error_k1 = error;
         
             /* add derivative portion to the output */
             tmp += portion; // P_portion + I_portion + D_portion
         }

         /* 5) Limit the controller output                      */
         if (tmp >= limit_positive) 
         { 
            output = limit_positive; 
            flags = SC_SATUR_POS_FLAG;
         }
         else if (tmp < limit_negative)
         {
            output = limit_negative; 
            flags = SC_SATUR_NEG_FLAG;
         }
         else
         {
            output = tmp;
            flags = 0;
         }
   
         /* 6) Save the controller output                          */
         *p_output = output<<8;
      }
      else
      {
         output = omega_desired_ramp;
         *p_output = output;
      }

      //**********************************************
      //* Interrupt each services_per_irq period.
      //**********************************************
      if ( --service_counter == 0 )
      {
         SetChannelInterrupt();
         service_counter = services_per_irq;  // Reset service counter.
      }

      LinkTo(link_chan);
      ClearLSRLatch();
      Pin(SetLow);
   }
/*******************************************************************************
* THREAD NAME: CALC_QD
* DESCRIPTION: Perform all SC calculations. The speed calculation is based on 
*              QD parameters.
*              1. Set pin high.
*              2. Actual speed calculation based on QD parameters.
*              3. Adjustment of desired speed to speed from defined ramp.
*              4. PID Controller calculations.
*              5. Generate Interrupt each services_per_irq period.
*              6. Send link to link_chan.
*              7. Clear link service latch.
*              8. Set pin low.
*******************************************************************************/
   else if (((link==1) && (flag0==1)) || 
            ((hsr==SC_HSR_CALC) && (flag0==1)))
   {
SC_CALC_QD:
      Pin(SetHigh);
    
      //**********************************************
      //* Speed Calculation.
      //**********************************************
      qd_position_difference = *p_qd_pc - qd_last_pc;
      qd_time_difference = *p_qd_tcr_value - qd_last_tcr;
          
      qd_last_pc=*p_qd_pc;
      qd_last_tcr=*p_qd_tcr_value;

      /* omega_actual=(scale*qd_position_difference)/qd_time_difference */
      tmp = scale;
      if (qd_position_difference < 0)
      {
         tmp = -tmp;
      }
      portion = ((qd_position_difference * tmp)/qd_time_difference)<<16;
      /* to raise the accuracy, divide the 8 bits left shifted reminder in mach
         by qd_time_difference; the result must be shifted 8 bits left and
         added to previous rough result */
      portion += ((mach<<8)/qd_time_difference)<<8;
      
      if (qd_position_difference < 0)
      {
         portion = -portion;
      }
      omega_actual = portion;

      goto SC_CALC_COMMON;
   }       
/*******************************************************************************
* THREAD NAME: MATCH_HD
* DESCRIPTION: Schedule next periodic match and perform all SC calculations. 
*              The speed calculation is based on HD parameters.
*              1. Schedule next match.
*              2. Goto SC_CALC_HD
*******************************************************************************/
   else if ((m1==1) && (flag0==0))
   {
      erta = edge_time+period;
      edge_time = erta;
      ClearMatchALatch();
      EnableMatchA();

      goto SC_CALC_HD;
   }
/*******************************************************************************
* THREAD NAME: MATCH_QD
* DESCRIPTION: Schedule next periodic match and perform all SC calculations. 
*              The speed calculation is based on QD parameters.
*              1. Schedule next match.
*              2. Goto SC_CALC_QD
*******************************************************************************/
   else if ((m1==1) && (flag0==1))
   {
      erta = edge_time+period;
      edge_time = erta;
      ClearMatchALatch();
      EnableMatchA();

      goto SC_CALC_QD;
   }
/*******************************************************************************
* THREAD NAME: UNHANDLED_EVENTS
*******************************************************************************/
   else 
   {
      #ifdef SET_USED  
      Global_Error_Func();             // Call the Global error routine and end.
      #endif
   }  
}



/*******************************************************************************
*  Information exported to Host CPU program.
*******************************************************************************/
#pragma write h, (::ETPUfilename (cpu/etpu_sc_auto.h));
#pragma write h, (/**************************************************************** );
#pragma write h, (* WARNING this file is automatically generated DO NOT EDIT IT!    );
#pragma write h, (*                                                                 );
#pragma write h, (* This file provides an interface between eTPU code and CPU       );
#pragma write h, (* code. All references to the SC function should be made with     );
#pragma write h, (* information in this file. This allows only symbolic             );
#pragma write h, (* information to be referenced which allows the eTPU code to be   );
#pragma write h, (* optimized without effecting the CPU code.                       );
#pragma write h, (*****************************************************************/);
#pragma write h, (#ifndef _ETPU_SC_AUTO_H_);
#pragma write h, (#define _ETPU_SC_AUTO_H_);
#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Function Configuration Information. );
#pragma write h, (****************************************************************/);
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_FUNCTION_NUMBER      ) SC_FUNCTION_NUMBER );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_TABLE_SELECT         ) ::ETPUentrytype(SC) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_NUM_PARMS            ) ::ETPUram(SC) );
#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Host Service Request Definitions. );
#pragma write h, (****************************************************************/);
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_INIT_HD_MASTER       ) SC_HSR_INIT_HD_MASTER);
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_INIT_HD_SLAVE        ) SC_HSR_INIT_HD_SLAVE );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_INIT_QD_MASTER       ) SC_HSR_INIT_QD_MASTER);
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_INIT_QD_SLAVE        ) SC_HSR_INIT_QD_SLAVE );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_CALC                 ) SC_HSR_CALC );
#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Parameter Definitions. );
#pragma write h, (****************************************************************/);
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_OPTIONS_OFFSET           ) ::ETPUlocation (SC, options) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_PERIOD_OFFSET            ) ::ETPUlocation (SC, period) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_STARTOFFSET_OFFSET       ) ::ETPUlocation (SC, start_offset) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_SERVICESPERIRQ_OFFSET    ) ::ETPUlocation (SC, services_per_irq) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_OMEGAACTUAL_OFFSET       ) ::ETPUlocation (SC, omega_actual) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_OMEGADESIRED_OFFSET      ) ::ETPUlocation (SC, omega_desired) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_OMEGADESIREDRAMP_OFFSET  ) ::ETPUlocation (SC, omega_desired_ramp) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_PHDPERIOD_OFFSET         ) ::ETPUlocation (SC, p_hd_period) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_PHDDIRECTION_OFFSET      ) ::ETPUlocation (SC, p_hd_direction) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_MAXHDPERIOD_OFFSET       ) ::ETPUlocation (SC, max_hd_period) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_PQDPC_OFFSET             ) ::ETPUlocation (SC, p_qd_pc) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_PQDTCRVALUE_OFFSET       ) ::ETPUlocation (SC, p_qd_tcr_value) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_QDLASTPC_OFFSET          ) ::ETPUlocation (SC, qd_last_pc) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_QDLASTTCR_OFFSET         ) ::ETPUlocation (SC, qd_last_tcr) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_POUTPUT_OFFSET           ) ::ETPUlocation (SC, p_output) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_LINKCHAN_OFFSET          ) ::ETPUlocation (SC, link_chan) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_SCALE_OFFSET             ) ::ETPUlocation (SC, scale) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_RAMPINCRUP_OFFSET        ) ::ETPUlocation (SC, ramp_incr_up) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_RAMPINCRDOWN_OFFSET      ) ::ETPUlocation (SC, ramp_incr_down) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_FLAGS_OFFSET             ) ::ETPUlocation (SC, flags) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_PGAIN_OFFSET             ) ::ETPUlocation (SC, P_gain) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_IGAIN_OFFSET             ) ::ETPUlocation (SC, I_gain) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_DGAIN_OFFSET             ) ::ETPUlocation (SC, D_gain) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_IK1_OFFSET               ) ::ETPUlocation (SC, I_k1) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_LIMITPOSITIVE_OFFSET     ) ::ETPUlocation (SC, limit_positive) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_LIMITNEGATIVE_OFFSET     ) ::ETPUlocation (SC, limit_negative) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_OUTPUT_OFFSET            ) ::ETPUlocation (SC, output) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_ERROR_OFFSET             ) ::ETPUlocation (SC, error) );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_ERROR_K1_OFFSET          ) ::ETPUlocation (SC, error_k1) );
#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Options Definitions. );
#pragma write h, (****************************************************************/);
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_P_SCALE              ) SC_P_SCALE );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_I_SCALE              ) SC_I_SCALE );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_D_SCALE              ) SC_D_SCALE );
#pragma write h, ( );
#pragma write h, (/****************************************************************);
#pragma write h, (* Flags Definitions. );
#pragma write h, (****************************************************************/);
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_SATUR_POS_FLAG       ) SC_SATUR_POS_FLAG );
#pragma write h, (::ETPUliteral(#define FS_ETPU_SC_SATUR_NEG_FLAG       ) SC_SATUR_NEG_FLAG );
#pragma write h, ( );
#pragma write h, (#endif);

/*********************************************************************
 *
 * Copyright:
 *	Freescale Semiconductor, INC. All Rights Reserved.
 *  You are hereby granted a copyright license to use, modify, and
 *  distribute the SOFTWARE so long as this entire notice is
 *  retained without alteration in any modified and/or redistributed
 *  versions, and that such modified versions are clearly identified
 *  as such. No licenses are granted by implication, estoppel or
 *  otherwise under any patents or trademarks of Freescale
 *  Semiconductor, Inc. This software is provided on an "AS IS"
 *  basis and without warranty.
 *
 *  To the maximum extent permitted by applicable law, Freescale
 *  Semiconductor DISCLAIMS ALL WARRANTIES WHETHER EXPRESS OR IMPLIED,
 *  INCLUDING IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A
 *  PARTICULAR PURPOSE AND ANY WARRANTY AGAINST INFRINGEMENT WITH
 *  REGARD TO THE SOFTWARE (INCLUDING ANY MODIFIED VERSIONS THEREOF)
 *  AND ANY ACCOMPANYING WRITTEN MATERIALS.
 *
 *  To the maximum extent permitted by applicable law, IN NO EVENT
 *  SHALL Freescale Semiconductor BE LIABLE FOR ANY DAMAGES WHATSOEVER
 *  (INCLUDING WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
 *  BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR OTHER
 *  PECUNIARY LOSS) ARISING OF THE USE OR INABILITY TO USE THE SOFTWARE.
 *
 *  Freescale Semiconductor assumes no responsibility for the
 *  maintenance and support of this software
 ********************************************************************/
